﻿using Rhino;
using Rhino.Collections;
using Rhino.FileIO;
using Rhino.PlugIns;

namespace TestKillwater
{
  public class TestKillwaterPlugIn : PlugIn
  {
    private const int MAJOR_VERSION = 1;
    private const int MINOR_VERSION = 0;

    /// <summary>
    /// Constructor
    /// </summary>
    public TestKillwaterPlugIn()
    {
      Instance = this;
      GlobalDatabase = new ArchivableDictionary(MAJOR_VERSION, "GlobalDatabase");
      LocalDatabase = new ArchivableDictionary(MAJOR_VERSION, "LocalDatabase");
    }

    #region Properties

    /// <summary>
    /// Gets the one and only instance of the TestKillwaterPlugIn object.
    /// </summary>
    public static TestKillwaterPlugIn Instance
    {
      get;
      private set;
    }

    /// <summary>
    /// Get the plug-in's global database
    /// </summary>
    public static ArchivableDictionary GlobalDatabase
    {
      get;
      private set;
    }

    /// <summary>
    /// Get the plug-in's local database
    /// </summary>
    public static ArchivableDictionary LocalDatabase
    {
      get;
      private set;
    }

    #endregion

    #region Members

    /// <summary>
    /// PlugIn.OnLoad override
    /// </summary>
    protected override LoadReturnCode OnLoad(ref string errorMessage)
    {
      Rhino.RhinoDoc.NewDocument += OnNewDocument;
      Rhino.RhinoDoc.BeginOpenDocument += OnBeginOpenDocument;
      return LoadReturnCode.Success;
    }

    private void ClearDatabases()
    {
      GlobalDatabase.Clear();
      LocalDatabase.Clear();
    }

    /// <summary>
    /// RhinoDoc.NewDocument event handler
    /// </summary>
    private void OnNewDocument(object sender, DocumentEventArgs e)
    {
      ClearDatabases();
    }

    /// <summary>
    /// RhinoDoc.OnBeginOpenDocument event handler
    /// </summary>
    private void OnBeginOpenDocument(object sender, DocumentOpenEventArgs e)
    {
      ClearDatabases();
    }

    /// <summary>
    /// PlugIn.ShouldCallWriteDocument override
    /// </summary>
    protected override bool ShouldCallWriteDocument(FileWriteOptions options)
    {
      return GlobalDatabase.Count > 0 || LocalDatabase.Count > 0;
    }

    /// <summary>
    /// PlugIn.WriteDocument override
    /// </summary>
    protected override void WriteDocument(RhinoDoc doc, BinaryArchiveWriter archive, FileWriteOptions options)
    {
      archive.Write3dmChunkVersion(MAJOR_VERSION, MINOR_VERSION);
      archive.WriteDictionary(GlobalDatabase);
      archive.WriteDictionary(LocalDatabase);
    }

    /// <summary>
    /// Plugin.ReadDocument override
    /// </summary>
    protected override void ReadDocument(RhinoDoc doc, BinaryArchiveReader archive, FileReadOptions options)
    {
      int major_version, minor_version;
      archive.Read3dmChunkVersion(out major_version, out minor_version);
      if (major_version == MAJOR_VERSION && minor_version == MINOR_VERSION)
      {
        GlobalDatabase = archive.ReadDictionary();
        LocalDatabase = archive.ReadDictionary();
      }
    }

    #endregion
  }
}